package jpatest.test;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.CacheRetrieveMode;
import javax.persistence.CacheStoreMode;
import javax.persistence.EntityGraph;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.NamedAttributeNode;
import javax.persistence.NamedEntityGraph;
import javax.persistence.NamedSubgraph;
import javax.persistence.Parameter;
import javax.persistence.PersistenceUnitUtil;
import javax.persistence.Query;
import javax.persistence.TemporalType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;

import org.junit.Test;

import jpatest.eao.BaseEAO;
import jpatest.eao.BookCategoryEAO;
import jpatest.eao.CustomerEAO;
import jpatest.eao.OrderEAO;
import jpatest.entity.Address;
import jpatest.entity.Address2;
import jpatest.entity.Address3;
import jpatest.entity.BankInfo;
import jpatest.entity.Book;
import jpatest.entity.Category;
import jpatest.entity.Customer;
import jpatest.entity.Customer5;
import jpatest.entity.CustomerType;
import jpatest.entity.GoldCustomer;
import jpatest.entity.GoldCustomer5;
import jpatest.entity.LineItem;
import jpatest.entity.Order;
import jpatest.entity.OrderDetail;
import jpatest.entity.OrderStatus;
import jpatest.entity.PreferredCustomer;
import jpatest.entity.PreferredCustomer5;

/**
 * This class tests entity graphs
 * 
 */
public class EntityGraphTest extends MyJpaTestCase {
	@Test
	public void testQueries() {
		try {
			QueryTest.createEntities(em);

			runBookEG();
			//runCustomerEG();
			//runOrderEG();
			//runOrderCustomerPhoneEG();

			// must delete at the end
			QueryTest.deleteEntities(em);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	protected void runBookEG() {
		System.out.flush();
		System.out.println("runnig runBookEG(): ");

		// fetchgraph for bookEG
		{
			EntityGraph bookGraph = em.getEntityGraph("bookEG");
			Book bk = (Book) em.createQuery("Select b from Book b")
					.setHint("javax.persistence.fetchgraph", bookGraph)
					.getResultList().get(0);
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();

			System.out.println("isbn:" + util.isLoaded(bk, "isbn"));
			System.out.println("author:" + util.isLoaded(bk, "author"));
			System.out.println("lineItems:" + util.isLoaded(bk, "lineItems"));
			System.out.println("categories:" + util.isLoaded(bk, "categories"));

			Category cy = bk.getCategories().iterator().next();
			System.out.println("categoryName:"
					+ util.isLoaded(cy, "categoryName"));
			System.out.println("parentCategory:"
					+ util.isLoaded(cy, "parentCategory"));
			System.out.println("childCategories:"
					+ util.isLoaded(cy, "childCategories"));
		}

		// loadgraph for bookEG
		{
			EntityGraph bookGraph = em.getEntityGraph("bookEG");
			Book bk = (Book) em.createQuery("Select b from Book b")
					.setHint("javax.persistence.loadgraph", bookGraph)
					.getResultList().get(0);
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();

			System.out.println("isbn:" + util.isLoaded(bk, "isbn"));
			System.out.println("author:" + util.isLoaded(bk, "author"));
			System.out.println("lineItems:" + util.isLoaded(bk, "lineItems"));
			System.out.println("categories:" + util.isLoaded(bk, "categories"));

			Category cy = bk.getCategories().iterator().next();
			System.out.println("categoryName:"
					+ util.isLoaded(cy, "categoryName"));
			System.out.println("parentCategory:"
					+ util.isLoaded(cy, "parentCategory"));
			System.out.println("childCategories:"
					+ util.isLoaded(cy, "childCategories"));
		}
		{ // dynamic creation of equivalent to bookEG
			EntityGraph bookGraph = em.createEntityGraph(Book.class);
			bookGraph.addAttributeNodes("isbn");
			bookGraph.addAttributeNodes("lineItems");
			bookGraph.addSubgraph("categories").addAttributeNodes(
					"parentCategory");

			Book bk = (Book) em.createQuery("Select b from Book b")
					.setHint("javax.persistence.fetchgraph", bookGraph)
					.getResultList().get(0);
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();

			System.out.println("isbn:" + util.isLoaded(bk, "isbn"));
			System.out.println("author:" + util.isLoaded(bk, "author"));
			System.out.println("lineItems:" + util.isLoaded(bk, "lineItems"));
			System.out.println("categories:" + util.isLoaded(bk, "categories"));
		}
		{ // dynamic modification of named entity graph bookEG
			EntityGraph bookGraph = em.createEntityGraph("bookEG");
			bookGraph.addAttributeNodes("author");
			bookGraph.addAttributeNodes("rating");
			bookGraph.addSubgraph("lineItems").addAttributeNodes(
					"quantity");

			Book bk = (Book) em.createQuery("Select b from Book b")
					.setHint("javax.persistence.fetchgraph", bookGraph)
					.getResultList().get(0);
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();

			System.out.println("isbn:" + util.isLoaded(bk, "isbn"));
			System.out.println("author:" + util.isLoaded(bk, "author"));
			System.out.println("lineItems:" + util.isLoaded(bk, "lineItems"));
			System.out.println("categories:" + util.isLoaded(bk, "categories"));
		}

	}

	protected void runCustomerEG() {
		System.out.flush();
		System.out.println("runnig runCustomerEG(): ");

		{ // customerEG
			EntityGraph custGraph = em.getEntityGraph("customerEG");
			Customer cust0 = (Customer) em
					.createQuery("Select c from Customer c")
					.setHint("javax.persistence.fetchgraph", custGraph)
					.getResultList().get(0);
			
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();
			System.out.println("phones:" + util.isLoaded(cust0, "phones"));
			System.out.println("orders:" + util.isLoaded(cust0, "orders"));
		}
		{ // customerEG
			EntityGraph custGraph = em.getEntityGraph("customerEG");
			Customer cust0 = (Customer) em
					.createQuery("Select c from Customer c")
					.setHint("javax.persistence.loadgraph", custGraph)
					.getResultList().get(0);
			
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();
			System.out.println("phones:" + util.isLoaded(cust0, "phones"));
			System.out.println("orders:" + util.isLoaded(cust0, "orders"));

		}
		{ // dynamic customerEG
			EntityGraph custGraph = em.createEntityGraph(Customer.class);
			custGraph.addAttributeNodes("phones");
			custGraph.addSubclassSubgraph(PreferredCustomer.class)
					.addAttributeNodes("expirationDate");
			Customer cust0 = (Customer) em
					.createQuery("Select c from Customer c")
					.setHint("javax.persistence.loadgraph", custGraph)
					.getResultList().get(0);
			
			PersistenceUnitUtil util = em.getEntityManagerFactory()
					.getPersistenceUnitUtil();
			System.out.println("phones:" + util.isLoaded(cust0, "phones"));
			System.out.println("orders:" + util.isLoaded(cust0, "orders"));
		}

	}

	protected void runOrderEG() {
		System.out.flush();
		System.out.println("runnig runOrderEG(): ");

		{ // orderEG
			EntityGraph orderGraph = em.getEntityGraph("orderEG");
			Order ord0 = (Order) em.createQuery("Select o from Order o")
					.setHint("javax.persistence.fetchgraph", orderGraph)
					.getResultList().get(0);
		}
		{ // orderEG
			EntityGraph orderGraph = em.getEntityGraph("orderEG");
			Order ord0 = (Order) em.createQuery("Select o from Order o")
					.setHint("javax.persistence.loadgraph", orderGraph)
					.getResultList().get(0);
		}
		{ // dynamic orderEG
			EntityGraph orderGraph = em.createEntityGraph(Order.class);
			orderGraph.addAttributeNodes("price");
			orderGraph.addAttributeNodes("lineItems");
			orderGraph.addSubgraph("customer").addAttributeNodes("bankInfos");
			orderGraph.addSubgraph("customer", GoldCustomer.class)
					.addAttributeNodes("creditLimit");
			Order ord0 = (Order) em.createQuery("Select o from Order o")
					.setHint("javax.persistence.loadgraph", orderGraph)
					.getResultList().get(0);
		}

	}

	protected void runOrderCustomerPhoneEG() {
		System.out.flush();
		System.out.println("runnig runOrderCustomerPhoneEG(): ");

		{ // orderCustomerPhoneEG
			EntityGraph ordercpGraph  = em.getEntityGraph("orderCustomerPhoneEG");
			Order ord2 = (Order) em.createQuery("Select o from Order o")
					.setHint("javax.persistence.fetchgraph", ordercpGraph )
					.getResultList().get(0);
		}
		{ // orderCustomerPhoneEG
			EntityGraph ordercpGraph  = em.getEntityGraph("orderCustomerPhoneEG");
			Order ord2 = (Order) em.createQuery("Select o from Order o")
					.setHint("javax.persistence.loadgraph", ordercpGraph )
					.getResultList().get(0);
		}
		{ // dynamic orderCustomerPhoneEG
			EntityGraph ordercpGraph  = em.createEntityGraph(Order.class);
			ordercpGraph.addAttributeNodes("lineItems");
			ordercpGraph.addSubgraph("customer").addSubgraph("phones")
					.addAttributeNodes("number");

			Order ord2 = (Order) em.createQuery("Select o from Order o")
					.setHint("javax.persistence.loadgraph", ordercpGraph)
					.getResultList().get(0);

		}

	}

}
