package jpatest.test;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.eclipse.persistence.config.PersistenceUnitProperties;

public class SchemaGen {

	static void generateSchemaFromPersistence() {
		Map<String, String> prop = new HashMap<String, String>();
		prop.put("javax.persistence.database-product-name", "Oracle");
		prop.put("javax.persistence.database-major-version", "12");
		prop.put("javax.persistence.database-minor-version", "1");
		prop.put("javax.persistence.schema-generation.scripts.action",
				"drop-and-create");
		prop.put("javax.persistence.schema-generation.create-source",
				"metadata");
		prop.put("javax.persistence.schema-generation.drop-source", "metadata");
		prop.put("javax.persistence.schema-generation.scripts.drop-target",
				"C:/Temp/generated_schema_Oracle_drop.sql");
		prop.put("javax.persistence.schema-generation.scripts.create-target",
				"C:/Temp/generated_schema_Oracle_create.sql");

		// Create database schemas and/or tables and/or create DDL scripts
		// Specify the persistence unit name as first parameter.
		// Called by the Persistence class when schema generation is to
		// occur as a separate phase from creation of the entity manager
		// factory.
		Persistence.generateSchema("jpaTestPU", prop);

	}

	static void generateSchemaFromEntityManagerFactory() throws IOException {

		Map prop = new HashMap();
		prop.put("javax.persistence.database-product-name", "Oracle");
		prop.put("javax.persistence.database-major-version", 12);
		prop.put("javax.persistence.database-minor-version", 1);
		prop.put("javax.persistence.schema-generation.scripts.action",
				"drop-and-create");
		prop.put("javax.persistence.schema-generation.create-source",
				"metadata");
		prop.put("javax.persistence.schema-generation.drop-source", "metadata");
		prop.put("javax.persistence.schema-generation.scripts.drop-target",
				new FileWriter(new File(
						"C:/Temp/generated_ddl_Oracle_drop.sql")));
		prop.put("javax.persistence.schema-generation.scripts.create-target",
				new FileWriter(new File(
						"C:/Temp/generated_ddl_Oracle_create.sql")));

		EntityManagerFactory emf = Persistence.createEntityManagerFactory(
				"jpaTestPU", prop);

	}
	
	static void generateSchemaFromEntityManagerFactoryEclipse() throws IOException {

		Map prop = new HashMap();
		
		prop.put(PersistenceUnitProperties.SCHEMA_GENERATION_DATABASE_ACTION, PersistenceUnitProperties.SCHEMA_GENERATION_DROP_AND_CREATE_ACTION);
		prop.put(PersistenceUnitProperties.SCHEMA_GENERATION_CREATE_SOURCE, PersistenceUnitProperties.SCHEMA_GENERATION_SCRIPT_SOURCE);
		prop.put(PersistenceUnitProperties.SCHEMA_GENERATION_CREATE_SCRIPT_SOURCE, new FileReader(new File("C:/Temp/mycreate.ddl")));
		prop.put(PersistenceUnitProperties.SCHEMA_GENERATION_DROP_SOURCE, PersistenceUnitProperties.SCHEMA_GENERATION_SCRIPT_SOURCE);
		prop.put(PersistenceUnitProperties.SCHEMA_GENERATION_DROP_SCRIPT_SOURCE, new FileReader(new File("C:/Temp/mydrop.ddl")));
		   

		EntityManagerFactory emf = Persistence.createEntityManagerFactory(
				"jpaTestPU", prop);

	}

	public static void main(String[] args) {
		try {

			 generateSchemaFromPersistence();

			//generateSchemaFromEntityManagerFactory();
			
			// using EclipseLink's constants
			//generateSchemaFromEntityManagerFactoryEclipse();
		} catch (Throwable t) {
			t.printStackTrace();
		}

	}

}
