package jpatest.test;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.ParameterMode;
import javax.persistence.Persistence;
import javax.persistence.StoredProcedureQuery;

import jpatest.entity.Address;
import jpatest.entity.BankInfo;
import jpatest.entity.Customer;
import jpatest.entity.CustomerType;
import jpatest.entity.GoldCustomer;
import jpatest.entity.Order;
import jpatest.entity.OrderStatus;
import jpatest.entity.PreferredCustomer;
import jpatest.util.StoredProcedures;

/**
 * JPA 2.1 adds support for stored procedures.
 */
public class StoredProcedureTest {
	// There is one entity manager factory per persistence unit
	EntityManagerFactory emf = null;

	public void test() {
		System.out.println("Run stored procedure Testing");
		try {

			// testing calling stored procedures using JDBC
			// jdbcSPTest();

			// test calling stored procedures using JPA 2.1
			jpaSPTest();

			// test stored procedure logic from outside database
			// testJavaStoredProcedureLogic();

		} catch (Throwable t) {
			t.printStackTrace();

		}
	}

	protected int insertCustomers(EntityManager em) {

		Order order = new Order();
		order.setOrderTime(new Timestamp(new Date().getTime()));
		order.setPrice(100.00);
		order.setStatus(OrderStatus.BILLED);

		Customer c = new Customer();
		c.setName("John Customer");
		c.setPicture("picture".getBytes());
		c.setCustomerType(CustomerType.C);
		order.setCustomer(c);

		Address ad = new Address();
		ad.setStreet("2920 Main Street");
		ad.setCity("LocalTown");
		ad.setState("MI");
		ad.setZip("18081");
		c.setAddress(ad);

		BankInfo bi = new BankInfo();
		bi.setBankName("NationalBank");
		bi.setRoutingNumber("1234567");
		bi.setAccountNumber("99887766-2");
		c.setBank(bi);

		PreferredCustomer p = new PreferredCustomer();
		p.setName("John Preferred");
		p.setPicture("picture".getBytes());
		p.setCustomerType(CustomerType.P);
		p.setDiscountRate(0.20);
		p.setExpirationDate(new Date());

		Order order2 = new Order();
		order2.setOrderTime(new Timestamp(new Date().getTime()));
		order2.setPrice(200.00);
		order2.setStatus(OrderStatus.BILLED);
		order.setCustomer(p);

		Address adp = new Address();
		adp.setStreet("2920 Peach Street");
		adp.setCity("Peach Town");
		adp.setState("MI");
		adp.setZip("18082");
		p.setAddress(adp);

		BankInfo bip = new BankInfo();
		bip.setBankName("National Peach Bank");
		bip.setRoutingNumber("1234568");
		bip.setAccountNumber("99887766-3");
		p.setBank(bip);

		GoldCustomer g = new GoldCustomer();
		g.setName("John Gold");
		g.setPicture("picture".getBytes());
		g.setCustomerType(CustomerType.G);
		g.setCardNumber("myCard#");

		Order order3 = new Order();
		order3.setOrderTime(new Timestamp(new Date().getTime()));
		order3.setPrice(200.00);
		order3.setStatus(OrderStatus.BILLED);
		order3.setCustomer(g);

		Address adg = new Address();
		adg.setStreet("2920 Gold Street");
		adg.setCity("Gold Town");
		adg.setState("MI");
		adg.setZip("18083");
		g.setAddress(adg);

		BankInfo big = new BankInfo();
		big.setBankName("National Gold Bank");
		big.setRoutingNumber("1234568");
		big.setAccountNumber("99887766-4");
		g.setBank(big);

		// create new customer, preferred customer, and gold customer
		em.getTransaction().begin();
		em.persist(c);
		em.persist(p);
		em.persist(g);
		em.persist(order);
		em.persist(order2);
		em.persist(order3);
		em.getTransaction().commit();

		int customerId = c.getCustomerId();

		return customerId;
	}

	public static void jdbcSPTest() throws SQLException {
		System.out.println("\ntesting using JDBC callable statement ");

		Order order = new Order();
		order.setOrderTime(new Timestamp(new Date().getTime()));
		order.setPrice(10.00);
		order.setStatus(OrderStatus.BILLED);

		Customer customer = new Customer();
		customer.setName("John Customer");
		customer.setPicture("picture".getBytes());
		customer.setCustomerType(CustomerType.C);
		order.setCustomer(customer);

		BankInfo bi = new BankInfo();
		bi.setBankName("NationalBank");
		bi.setRoutingNumber("1234567");
		bi.setAccountNumber("99887766-2");
		customer.setBank(bi);

		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("jpaTestPU");
		EntityManager em = emf.createEntityManager();

		em.getTransaction().begin();
		em.persist(customer);
		em.persist(order);
		em.getTransaction().commit();

		int customerId = customer.getCustomerId();
		em.close();
		emf.close();

		Connection conn = DriverManager.getConnection(
				"jdbc:derby://localhost:1527/jpatest", "jpatest", "jpatest");

		// call a stored procedure
		{
			CallableStatement cs = conn
					.prepareCall("{ call GET_AVG_PRICE(?, ?, ?)}");
			cs.setInt(1, customerId);
			cs.setBigDecimal(2, new BigDecimal(2.5));
			cs.registerOutParameter(2, java.sql.Types.DECIMAL);
			cs.registerOutParameter(3, java.sql.Types.VARCHAR);

			boolean status = cs.execute();
			BigDecimal avgPrice = cs.getBigDecimal(2);
			String custName = cs.getString(3);
			System.out.println("avg order price: " + avgPrice + " custName:"
					+ custName);
		}

		// call a stored procedure with a result set
		{
			CallableStatement cs = conn
					.prepareCall("{ call GET_CUSTOMER_ORDER(?, ?)}");
			cs.setInt(1, customerId);
			cs.setBigDecimal(2, new BigDecimal(2.5));
			cs.registerOutParameter(2, java.sql.Types.DECIMAL);

			boolean status = cs.execute();
			BigDecimal avgPrice = cs.getBigDecimal(2);

			System.out.println("2nd SP avg order price: " + avgPrice);

			ResultSet rs = cs.getResultSet();
			while (rs.next()) {
				System.out.println("2nd SP Result Set 1st column: "
						+ rs.getString(1));
			}

			// more result sets?
			boolean more = cs.getMoreResults();
			if (more) {
				System.out.println("There is another result set");
				rs = cs.getResultSet();
			}
		}

		// call a stored procedure that updates database and also retrieves
		// some data
		{
			CallableStatement cs = conn
					.prepareCall("{ call UPDATE_CUSTOMER(?, ?, ?)}");
			cs.setInt(1, customerId);
			cs.setString(2, " New Cust Name");
			cs.registerOutParameter(3, java.sql.Types.DOUBLE);

			int status = cs.executeUpdate();
			Double sumPrice = cs.getDouble(3);
			System.out.println("sum order price: " + sumPrice + " rowsUpdated:"
					+ status);
		}

	}

	// testing stored procedures in JPA 2.1
	public static void jpaSPTest() {
		System.out.println("\ntesting stored procedure feature of JPA 2.1");
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("jpaTestPU");

		int customerId = 0;
		{

			Order order = new Order();
			order.setOrderTime(new Timestamp(new Date().getTime()));
			order.setPrice(100.00);
			order.setStatus(OrderStatus.BILLED);

			Customer customer = new Customer();
			customer.setName("John Customer");
			customer.setPicture("picture".getBytes());
			customer.setCustomerType(CustomerType.C);
			order.setCustomer(customer);

			BankInfo bi = new BankInfo();
			bi.setBankName("NationalBank");
			bi.setRoutingNumber("1234567");
			bi.setAccountNumber("99887766-2");
			customer.setBank(bi);

			EntityManager em = emf.createEntityManager();

			em.getTransaction().begin();
			em.persist(customer);
			em.persist(order);
			em.getTransaction().commit();
			customerId = customer.getCustomerId();

			em.close();
		}
		{ // calling a named stored procedure query by its query name
			EntityManager em = emf.createEntityManager();
			StoredProcedureQuery spq = em
					.createNamedStoredProcedureQuery("NSPQ_GET_AVG_PRICE");
			spq.setParameter(1, customerId);
			spq.setParameter(2, new BigDecimal(5));

			spq.execute();
			BigDecimal avgPrice = (BigDecimal) spq.getOutputParameterValue(2);
			String custName = (String) spq.getOutputParameterValue(3);

			System.out.println("NSPQ_GET_AVG_PRICE returned avgPrice= "
					+ avgPrice + " custName = " + custName);

		}

		{ // calling a stored procedure by its name in the database
			EntityManager em = emf.createEntityManager();
			StoredProcedureQuery spq = em
					.createStoredProcedureQuery("GET_AVG_PRICE");
			spq.registerStoredProcedureParameter(1, Integer.class,
					ParameterMode.IN);
			spq.registerStoredProcedureParameter(2, BigDecimal.class,
					ParameterMode.INOUT);
			spq.registerStoredProcedureParameter(3, String.class,
					ParameterMode.OUT);

			spq.setParameter(1, customerId);
			spq.setParameter(2, new BigDecimal(5));

			spq.execute();

			BigDecimal avgPrice = (BigDecimal) spq.getOutputParameterValue(2);
			String custName = (String) spq.getOutputParameterValue(3);

			System.out.println("GET_AVG_PRICE returned avgPrice= " + avgPrice
					+ " custName = " + custName);

		}

		{ // calling a name stored procedure which returns a result set
			EntityManager em = emf.createEntityManager();
			StoredProcedureQuery spq = em
					.createNamedStoredProcedureQuery("NSPQ_GET_CUSTOMER_ORDER");
			spq.setParameter(1, customerId);
			spq.setParameter(2, null);

			boolean isResultset = spq.execute();
			// The execute method returns true if the first result is a result
			// set, and false if
			// it is an update count or there are no results other than through
			// INOUT and OUT
			// parameters, if any.
			// If the execute method returns true, the pending result set can be
			// obtained by
			// calling getResultList or getSingleResult.

			// For portability, results that correspond to JDBC result sets
			// and update counts need to be processed before the values of
			// any INOUT or OUT parameters are extracted.
			// After results returned through getResultList and
			// getUpdateCount have been exhausted, results returned through
			// INOUT and OUT parameters can be retrieved.
			if (isResultset) {
				List<Customer> customerList = spq.getResultList();
				for (Customer c : customerList) {
					System.out
							.println("NSPQ_GET_CUSTOMER_ORDER retrieved customer name: "
									+ c.getName());
				}
			}

			if (spq.hasMoreResults()) {
				// hasMoreResults() returns true if the next result
				// corresponds to a result set, and false if it is an update
				// count or if there are no results other than
				// through INOUT and OUT parameters, if any.
				System.out.println("There is more result set");
				spq.getResultList();
			}

			BigDecimal avgPrice = (BigDecimal) spq.getOutputParameterValue(2);
			System.out.println("NSPQ_GET_CUSTOMER_ORDER returned avgPrice= "
					+ avgPrice);
		}

		{ // calling a dynamic stored procedure which returns a result set
			EntityManager em = emf.createEntityManager();
			StoredProcedureQuery spq = em.createStoredProcedureQuery(
					"GET_CUSTOMER_ORDER", Customer.class);
			spq.registerStoredProcedureParameter(1, Integer.class,
					ParameterMode.IN);
			spq.registerStoredProcedureParameter(2, BigDecimal.class,
					ParameterMode.INOUT);
			spq.setParameter(1, customerId);
			spq.setParameter(2, null);

			spq.execute();

			List<Customer> customerList = spq.getResultList();
			for (Customer c : customerList) {
				System.out
						.println("GET_CUSTOMER_ORDER retrieved customer name: "
								+ c.getName());
			}

			if (spq.hasMoreResults()) {
				// hasMoreResults() returns true if the next result
				// corresponds to a result set, and false if it is an update
				// count or if there are no results other than
				// through INOUT and OUT parameters, if any.
				System.out.println("There is more result set");
				spq.getResultList();
			}

			BigDecimal avgPrice = (BigDecimal) spq.getOutputParameterValue(2);
			System.out.println("GET_CUSTOMER_ORDER returned avgPrice= "
					+ avgPrice);
		}

		{ // test an update stored procedure
			EntityManager em = emf.createEntityManager();
			StoredProcedureQuery spq = em
					.createStoredProcedureQuery("UPDATE_CUSTOMER");
			spq.registerStoredProcedureParameter(1, Integer.class,
					ParameterMode.IN);
			spq.registerStoredProcedureParameter(2, String.class,
					ParameterMode.IN);
			spq.registerStoredProcedureParameter(3, Double.class,
					ParameterMode.OUT);
			spq.setParameter(1, customerId);
			spq.setParameter(2, "New Cust Name");

			try {
				// it requires an active transaction for update/delete
				em.getTransaction().begin();
				int updateCount0 = spq.executeUpdate();
				em.getTransaction().commit();

				int updateCount = spq.getUpdateCount();
				spq.execute();
				Double sumPrice = (Double) spq.getOutputParameterValue(3);

				System.out.println("UPDATE_CUSTOMER returned sumPrice= "
						+ sumPrice + " with rows updated = " + updateCount);
			} catch (Throwable t) {
				t.printStackTrace();
				em.getTransaction().rollback();
			} finally {
				em.close();
			}
		}

		emf.close();

	}

	// Test Java Stored procedure, which shows that Java stored procedures are
	// easier to debug
	public void testJavaStoredProcedureLogic() {

		EntityManager em = emf.createEntityManager();
		try {

			int id = insertCustomers(em);

			{ // test logic in a java stored procedure method
				BigDecimal[] avgPrice = new BigDecimal[1];
				java.sql.ResultSet[] resultSet = new ResultSet[1];

				StoredProcedures.spCustomerAndPrice(id, avgPrice, resultSet);
				System.out.println("spCustomerAndPrice returned avgPrice= "
						+ avgPrice[0]);
				while (resultSet[0].next()) {
					System.out.println("customer column 1: "
							+ resultSet[0].getString(1));
				}
			}

			{ // test logic in a java stored procedure method
				BigDecimal[] avgPrice = new BigDecimal[1];
				String[] custName = new String[1];

				StoredProcedures.spAvgPriceName(id, avgPrice, custName);
				System.out.println("spAvgPriceName returned avgPrice= "
						+ avgPrice[0] + " custName = " + custName[0]);
			}

			{ // test logic in a java stored procedure method
				Double[] sumPrice = new Double[1];
				StoredProcedures.spUpdateCustomer(id, "newCustName", sumPrice);
				System.out.println("spUpdateCustomer returned sumPrice= "
						+ sumPrice[0]);
			}

		} catch (Throwable t) {
			t.printStackTrace();

		} finally {
			em.close();
		}
	}

	public StoredProcedureTest() {
		emf = Persistence.createEntityManagerFactory("jpaTestPU");
	}

	public void close() {
		if (emf != null && emf.isOpen()) {
			emf.close();
		}
	}

	public static void main(String[] args) {
		try {
			StoredProcedureTest spTest = new StoredProcedureTest();
			spTest.test();
			spTest.close();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

}
